/*
 * main.c
 *
 *  Created on: 23.07.2018
 *      Author: rieche
 */

/*******************************************************************************
* (c)Copyright 2016-2017 Microsemi Corporation.  All rights reserved.
*
*  Simple IOD CDR 1G loop back  example program .
*
*
*/

#include <stdio.h>
#include <string.h>

#include "drivers/CoreTSE/core_tse.h"
#include "drivers/CoreUARTapb/core_uart_apb.h"
#include "riscv_hal.h"
#include "hal.h"
#include "hw_platform.h"
#include "ZL30722.h"
#include "ZL30722_image.h"

#include "coretse_regs.h"

#include "uip/uip.h"
#include "uip/uip_arp.h"
#include "uip/uip-split.h"

#include "timer.h"


/* locale macros **************************************************************/
#define BUF ((struct uip_eth_hdr *)&uip_buf[0])

/* global data types **********************************************************/
// g_tse has to be located in un-cached memory (IO space)
tse_instance_t g_tse __attribute__ ((section(".uncached_data")));
UART_instance_t g_uart;
uint8_t g_uart_string[81] = {0};
static struct timer periodic_timer, arp_timer;

void print(uint8_t *string)
{
	UART_polled_tx_string(&g_uart, string);
}


/* locale functions ***********************************************************/


uint8_t External_2_IRQHandler(void)
{
	TSE_isr(&g_tse);
	PLIC_CompleteIRQ(External_2_IRQn);
	return(EXT_IRQ_KEEP_ENABLED);
}

static void tx_handler(void *caller_info)
{
	*((uint32_t *)caller_info) = 0;
}

static void rx_handler(uint8_t *p_rx_packet, uint32_t length, void *caller_info)
{
	uip_len = length;
}

static void display_link_status(void)
{
	uint8_t     link_up;
	uint8_t     fullduplex;
	tse_speed_t speed;

	link_up = TSE_get_link_status(
			&g_tse,
			(tse_speed_t *)&speed,
			(uint8_t *)&fullduplex
			);

	print("\r\nEthernet link ");

	if(1u == link_up) {
		print("up: ");

		switch(speed) {
		case TSE_MAC10MBPS:
			print("10Mbps ");
			break;

		case TSE_MAC100MBPS:
			print("100Mbps ");
			break;

		case TSE_MAC1000MBPS:
			print("1000Mbps ");
			break;

		default:
			break;
		}

		if(1u == fullduplex)
			print("Full");
		else
			print("Half");
		print(" Duplex\r\n");

	} else {
		print("down.");
	}
	print("\r\n");
}

static void init(void)
{
	static tse_cfg_t tse_config;
	const struct uip_eth_addr mac_address = {
		0xC0, 0xB1, 0x3C, 0x88, 0x88, 0x82
	};
	u16_t ipaddr[2];
	tse_speed_t speed;
	uint8_t fullduplex;

	PLIC_init();
	PLIC_SetPriority(External_2_IRQn, 1);
	PLIC_EnableIRQ(External_2_IRQn);
	PLIC_SetPriority(External_3_IRQn, 1);
	PLIC_EnableIRQ(External_3_IRQn);

	UART_init(
		&g_uart,
		COREUARTAPB0_BASE_ADDR,
		BAUD_VALUE_115200,
		(DATA_8_BITS | NO_PARITY)
	);

	print("uart init done...\n\r");

	ZL30722_SPI_init();
	/* ZL30722 already configured? */
	ZL30722_compare(ZL30722_image, ZL30722_IMAGE_SIZE);

	TSE_cfg_struct_def_init(&tse_config);

	tse_config.phy_addr = 0; //TSE_PHY_ADDRESS_AUTO_DETECT;
	tse_config.speed_duplex_select = TSE_ANEG_ALL_SPEEDS;
	tse_config.mac_addr[0] = mac_address.addr[0];
	tse_config.mac_addr[1] = mac_address.addr[1];
	tse_config.mac_addr[2] = mac_address.addr[2];
	tse_config.mac_addr[3] = mac_address.addr[3];
	tse_config.mac_addr[4] = mac_address.addr[4];
	tse_config.mac_addr[5] = mac_address.addr[5];
	tse_config.framefilter = TSE_FC_PROMISCOUS_MODE_MASK;

	TSE_init(&g_tse, TSE_BASEADDR, &tse_config);

	HAL_enable_interrupts();

	if(TSE_get_link_status(&g_tse, &speed, &fullduplex) == TSE_LINK_UP) {
		TSE_set_tx_callback(&g_tse, tx_handler);
		TSE_set_rx_callback(&g_tse, rx_handler);
		TSE_receive_pkt(&g_tse, uip_buf, 0);

		/* ethernet timer */
		timer_init();
		timer_set(&periodic_timer, CLOCK_SECOND / 2);
		timer_set(&arp_timer, CLOCK_SECOND * 10);

		uip_setethaddr(mac_address);
		uip_init();
		uip_arp_init();
#if 0
		dhcpc_init(&mac_address.addr[0], 6);
#else
//		uip_ipaddr(ipaddr, 172, 16, 2, 75);
		uip_ipaddr(ipaddr, 169, 254, 97, 129);
		uip_sethostaddr(ipaddr);

//		uip_ipaddr(ipaddr, 172, 16, 2, 15);
		uip_ipaddr(ipaddr, 169, 254, 97, 130);
		uip_setdraddr(ipaddr);

		uip_ipaddr(ipaddr, 255, 255, 0, 0);
		uip_setnetmask(ipaddr);
#endif
		httpd_init();
	}
	display_link_status();
}

/* main ***********************************************************************/
int main()
{
	init();

	while(1) {
		if(uip_len > 0) {
		      if(BUF->type == htons(UIP_ETHTYPE_IP)) {
			      uip_arp_ipin();
			      uip_input();
			      if(uip_len > 0) {
				      uip_arp_out();
				      TSE_send_pkt(&g_tse, uip_buf, uip_len, 0);
			      }
		      } else if(BUF->type == htons(UIP_ETHTYPE_ARP)) {
			      uip_arp_arpin();
			      if(uip_len > 0) {
				      TSE_send_pkt(&g_tse, uip_buf, uip_len, 0);
			      }
		      }
		} else if(timer_expired(&periodic_timer)) {
			int i;
			timer_reset(&periodic_timer);
			for(i = 0; i < UIP_CONNS; i++) {
				uip_periodic(i);
				if(uip_len > 0) {
					uip_arp_out();
					TSE_send_pkt(&g_tse, uip_buf, uip_len, 0);
				}
			}

#if UIP_UDP
			for(i = 0; i < UIP_UDP_CONNS; i++) {
				uip_udp_periodic(i);
				if(uip_len > 0) {
					uip_arp_out();
					TSE_send_pkt(&g_tse, uip_buf, uip_len, 0);
				}
			}
#endif /* UIP_UDP */

			/* Call the ARP timer function every 10 seconds. */
			if(timer_expired(&arp_timer)) {
				timer_reset(&arp_timer);
				uip_arp_timer();
			}
		}
		TSE_receive_pkt(&g_tse, uip_buf, 0);
	}
}
